#-------------------------------------------------------------------------------
# Plugins::GMail::ClamAV_Attach_Scan - Install.pm module
#
# Copyright (c) 2004 Charlie Piper, x2concepts.com
#-------------------------------------------------------------------------------
# Author    : Charlie Piper <piper@x2concepts.com>
# Date      : Jul 10, 2004
# CVS       : $Id: Install.pm,v 1.3 2004/07/24 05:32:40 piper Exp $
#-------------------------------------------------------------------------------
package Plugins::GMail::ClamAV_Attach_Scan;
#-------------------------------------------------------------------------------

use strict;
use vars  qw/$DEBUG $NAME $META/;
use GT::Base;
use GT::Plugins qw/STOP CONTINUE/;
use GMail;

$DEBUG   = 0;
$NAME    = 'ClamAV_Attach_Scan';

# Inhert from base class for debug and error methods
@Plugins::GMail::ClamAV_Attach_Scan::ISA = qw(GT::Base);

$META = {
    'url' => 'http://www.x2concepts.com/',
    'description' => 'Scan attachments with ClamAV virus scanner.  Requires 
                      ClamAV to be installed and running.',
    'author'      => 'Charlie Piper',
    'license'     => 'GPL',
    'version'     => 1.1,
    'prog_ver'    => '2.2.3-Upgrade',
    'last_modified' => '2004/07/21 11:40',
};


sub pre_install {
# -----------------------------------------------------------------------------
# This function displays an HTML formatted message that will display any
# instructions/information to the user before they install the plugin.
#
    return q~You are about to install the ClamAV Attachemnt Scan Plugin.  Please 
      refer to the info link upon installation to complete the setup.~;
}

sub pre_uninstall {
# -----------------------------------------------------------------------------
# This function displays an HTML formatted message that will display any
# instructions/information to the user before they remove the plugin.
#
    return 'You are about to uninstall the ClamAV Attachemnt Scan Plugin.';
}

sub install {
# -----------------------------------------------------------------------------
# This function does the actual installation.  Its first argument is a plugin
# manager which you can use to register hooks, install files, add menu options,
# etc.  The second argument is a GT::Tar object which you can use to access any
# files in your plugin module.
#
# You should return an HTML formatted string that will be displayed to the
# user.
#
# If there is an error, return undef, and set the error message in
# $Plugins::GMail::ClamAV_Attach_Scan::error
#
    my ($mgr, $tar) = @_;
    
    $mgr->install_hooks('ClamAV_Attach_Scan', [['GMail::Email::Attach::upload', 'POST', 'Plugins::GMail::ClamAV_Attach_Scan::scan', 'FIRST']]);
    $mgr->install_menu('ClamAV_Attach_Scan', [['Info', 'admin.cgi?do=plugin&plugin=ClamAV_Attach_Scan&func=info']]);
    $mgr->install_options('ClamAV_Attach_Scan', [['ClamAV Port', '/tmp/clamd', 'This is the port or socket on which ClamAV is running.  Check your clamav.conf if you are unsure.', '', [], [], '']]);

    return "The ClamAV Attachment Scan Plugin has been successfully installed!";
}

sub uninstall {
# -----------------------------------------------------------------------------
# This function removes the plugin.  Its first argument is also a plugin
# manager which you can use to register hooks, install files, add menu options,
# etc. You should return an HTML formatted string that will be displayed to the
# user.
#
# If there is an error, return undef, and set the error message in
# $Plugins::GMail::ClamAV_Attach_Scan::error
#
    my $mgr = shift;
    
    $mgr->uninstall_hooks('ClamAV_Attach_Scan', [['GMail::Email::Attach::upload', 'POST', 'Plugins::GMail::ClamAV_Attach_Scan::scan', 'FIRST']]);
    $mgr->uninstall_menu('ClamAV_Attach_Scan', [['Info', 'admin.cgi?do=plugin&plugin=ClamAV_Attach_Scan&func=info']]);
    $mgr->install_options('ClamAV_Attach_Scan', [['ClamAV Port', '/tmp/clamd', 'This is the port or socket on which ClamAV is running.  Check your clamav.conf if you are unsure.', '', [], [], '']]);

    return "The ClamAV Attachment Scan Plugin has been successfully removed!";
}

1;